@@ -168,9 +168,9 @@ class YOLO(object):
             class_confidence = box_confidence * class probability
         and a multiclass non-max suppression with iou_threshold and a maximal
         number of returning boundind boxes.
-        If text_format is true, this method returns a cl output TODO.
+        If text_format is true, this method returns a dict.
         If text_format is false, this method return the input image with
-        its predicted bounding boxes.
+        its predicted bounding boxes (tensor).
 
         Args:
             img: Image to perform prediction on.
@@ -190,14 +190,15 @@ class YOLO(object):
             class_conf_threshold=class_conf_threshold)
 
         if text_format:
+            result = []
             for obj, cl, box in zip(selected_confidence, selected_classes,
                                     selected_boxes):
-                print("Confidence: {}".format(obj))
-                print("Class: {}".format(self.class_list[tf.cast(
-                    cl, dtype=tf.int32)]))
-                print("Box: {}".format(box))
-                print("")
-                # TODO consitent return.
+                item = {}
+                item["Class"] = self.class_list[tf.cast(cl, dtype=tf.int32)]
+                item["Confidence"] = obj
+                item["Box"] = box
+                result.append(item)
+            return result
         else:
             # Swap xy coordinates for bullshit tensorflow draw_bounding_boxes
             xmin = selected_boxes[..., 0:1]
@@ -327,13 +328,12 @@ class YOLO(object):
                     lambda_coord, lambda_obj, lambda_noobj, lambda_class,
                     iou_threshold, class_conf_threshold)
 
-                train_loss_avg(loss)
+                train_loss_avg.update_state(loss)
 
                 # TODO Workaround. summary image & gpu usage.
                 with file_writer.as_default():
                     tf.summary.image("label", tb_label, step=step)
-                    tf.summary.image(
-                        "prediction_training", tb_prediction, step=step)
+                    tf.summary.image("training", tb_prediction, step=step)
 
                 file_writer.flush()
                 progbar.update(current=(step_float))
@@ -350,131 +350,20 @@ class YOLO(object):
                 epoch + 1, round(train_loss_avg.result().numpy().item(), 4)))
 
             print("Start validation loop...")
-            # Run a validation loop at the end of each epoch.
-            groundtruth_annotations_list = []
-            img_list = []
-            detections_list = []
-            img_id = 1
-            val_loss_avg = tf.keras.metrics.Mean()
-            for elements in validation_data:
-                images = elements[0]
-                labels = elements[1]
-                # TODO order?
-                img_width = int(
-                    tf.cast(tf.shape(images)[1], dtype=tf.float32).numpy())
-                img_height = int(
-                    tf.cast(tf.shape(images)[2], dtype=tf.float32).numpy())
-                # Set training false, so that BN uses the mean and variance of
-                # its moving statistics, learned during training.
-                val_logits = self.model(images, training=True)
-                loss = yolo_loss(
-                    labels,
-                    val_logits,
-                    self.anchors,
-                    step=step,
-                    is_training=False)
-                val_loss_avg(loss)
-
-                # Validation prediction
-                # Bounding box prediction to tensorboard.
-                tb_prediction_val = self._prediction_to_tensorboard(
-                    images, val_logits, max_outputs, step, iou_threshold,
-                    class_conf_threshold)
-                with eval_file_writer.as_default():
-                    tf.summary.image(
-                        "prediction_validation", tb_prediction_val, step=step)
-
-                # Coco mAP validation metrics.
-                # Effective batch size. Last batch could contain fewer elements.
-                eff_bs = tf.cast(
-                    tf.shape(images)[0], dtype=tf.int32).numpy().item()
-                bboxs, cat_ids, scores = self.coco_metric_format(
-                    val_logits, eff_bs, class_conf_threshold, iou_threshold,
-                    False)
-                # Iterate over all detections per image.
-                for bbox, cat_id, score in zip(bboxs, cat_ids, scores):
-                    # bbox is: [xmin,ymin,xmax,ymax]
-                    # bbox : [x,y,w,h]
-                    xmin = bbox[0]
-                    ymin = bbox[1]
-                    xmax = bbox[2]
-                    ymax = bbox[3]
-
-                    # Center coordinate.
-                    x = 0.5 * (xmin + xmax) * img_width
-                    y = 0.5 * (ymin + ymax) * img_height
-                    width = (xmax - xmin) * img_width
-                    height = (ymax - ymin) * img_height
-
-                    detect_item = {
-                        'image_id': img_id,
-                        'category_id': int(cat_id.item()),
-                        'bbox': [x, y, width, height],
-                        'score': score
-                    }
-                    detections_list.append(detect_item)
-                # Iterate iver all objects in labels.
-                bboxs, cat_ids, scores = self.coco_metric_format(
-                    labels, eff_bs, 0, iou_threshold, True)
-                obj_id = 1
-                for bbox, cat_id, score in zip(bboxs, cat_ids, scores):
-                    # bbox is: [xmin,ymin,xmax,ymax]
-                    # bbox : [x,y,w,h]
-                    xmin = bbox[0]
-                    ymin = bbox[1]
-                    xmax = bbox[2]
-                    ymax = bbox[3]
-
-                    # Center coordinate.
-                    x = 0.5 * (xmin + xmax) * img_width
-                    y = 0.5 * (ymin + ymax) * img_height
-                    width = (xmax - xmin) * img_width
-                    height = (ymax - ymin) * img_height
-
-                    grountruth_item = {
-                        'id': obj_id,
-                        'image_id': img_id,
-                        'category_id': int(cat_id.item()),
-                        'bbox': [x, y, width, height],
-                        'area': width * height,
-                        'iscrowd': 0
-                    }
-                    obj_id += 1
-                    img_list.append({"id": img_id})
-
-                    groundtruth_annotations_list.append(grountruth_item)
-
-                img_id += 1
-
-            groundtruth_dict = {
-                'annotations': groundtruth_annotations_list,
-                'images': img_list,
-                'categories': self.category_list
-            }
-            #print("gt", groundtruth_dict)
-            #print("p", detections_list)
-            metrics = compute_coco_metrics(groundtruth_dict, detections_list)
-            with eval_file_writer.as_default():
-                tf.summary.scalar(
-                    "total_loss", val_loss_avg.result(), step=step)
-                for key in metrics:
-                    tf.summary.scalar(key, metrics[key], step=step)
-            eval_file_writer.flush()
-
-            if mlflow:
-                # Convert to native python type (db).
-                mlflow.log_metric(
-                    "mAP50IOU",
-                    metrics["Precision/mAP@.50IOU"].item(),
-                    step=step_float)
-                mlflow.log_metric(
-                    "AR100", metrics["Recall/AR@100"].item(), step=step_float)
-                mlflow.log_metric(
-                    "val_loss",
-                    val_loss_avg.result().numpy().item(),
-                    step=step_float)
-            print("...Done - loss {}\n".format(
-                round(val_loss_avg.result().numpy().item(), 4)))
+            self._evaluate(
+                validation_data,
+                iou_threshold=iou_threshold,
+                class_conf_threshold=class_conf_threshold,
+                mlflow=mlflow,
+                tb=True,
+                lambda_coord=lambda_coord,
+                lambda_obj=lambda_obj,
+                lambda_noobj=lambda_noobj,
+                lambda_class=lambda_class,
+                max_outputs=max_outputs,
+                step=step,
+                step_float=step_float,
+                file_writer=eval_file_writer)
 
     def coco_metric_format(self, tensor, batch_size, class_conf_threshold,
                            iou_threshold, label):
@@ -526,9 +415,10 @@ class YOLO(object):
             scores.append(selected_confidence.numpy())
 
         # Flatten nested lists.
-        bboxs = [item for sublist in bboxs for item in sublist]
-        cat_ids = [item for sublist in cat_ids for item in sublist]
-        scores = [item for sublist in scores for item in sublist]
+        #bboxs = [item for sublist in bboxs for item in sublist]
+        #cat_ids = [item for sublist in cat_ids for item in sublist]
+        #print(scores)
+        #scores = [item for sublist in scores for item in sublist]
         return bboxs, cat_ids, scores
 
     def restore(self, ckpt_path):
@@ -586,7 +476,8 @@ class YOLO(object):
                 prediction = self.model(x, training=True)
                 end_time = time.time()
                 img_per_second = (end_time - start_time) / batch_size
-                tf.summary.scalar("img/sec", img_per_second, step=step)
+                tf.summary.scalar(
+                    "inference_time[s]", img_per_second, step=step)
 
                 # Bounding box prediction to tensorboard.
                 tb_prediction = self._prediction_to_tensorboard(
@@ -611,6 +502,7 @@ class YOLO(object):
                     lambda_obj,
                     lambda_noobj,
                     lambda_class,
+                    iou_threshold=iou_threshold,
                     step=step)
 
             # Calculate gradient.
@@ -623,15 +515,33 @@ class YOLO(object):
             # Piece of shit workaround
             return tb_prediction, tb_label, loss
 
-    def evaluate(self, dataset, batch_size, verbose=1):
-        """TODO(): Returns loss and metrics (map, recall) for object detection.
+    def evaluate(self,
+                 dataset,
+                 iou_threshold=0.4,
+                 class_conf_threshold=0.25,
+                 lambda_coord=1.0,
+                 lambda_obj=5.0,
+                 lambda_noobj=1.0,
+                 lambda_class=1.0):
+        """[summary]
 
         Args:
             dataset ([type]): [description]
-            batch_size ([type]): [description]
-            verbose (int, optional): [description]. Defaults to 1.
+            iou_threshold (float, optional): [description]. Defaults to 0.4.
+            class_conf_threshold (float, optional): [description]. Defaults to 0.25.
+            lambda_coord (float, optional): [description]. Defaults to 1.0.
+            lambda_obj (float, optional): [description]. Defaults to 5.0.
+            lambda_noobj (float, optional): [description]. Defaults to 1.0.
+            lambda_class (float, optional): [description]. Defaults to 1.0.
         """
-        pass
+        self._evaluate(
+            dataset,
+            iou_threshold=iou_threshold,
+            class_conf_threshold=class_conf_threshold,
+            lambda_coord=lambda_coord,
+            lambda_obj=lambda_obj,
+            lambda_noobj=lambda_noobj,
+            lambda_class=lambda_class)
 
     def decode_yolo_output(self,
                            predictions,
@@ -798,3 +708,159 @@ class YOLO(object):
         tb_output = tf.concat(tb_imgs, axis=0)
         #tf.summary.image("prediction", tb_output, step=step)
         return tb_output
+
+    def _evaluate(self,
+                  dataset,
+                  iou_threshold=0.4,
+                  class_conf_threshold=0.25,
+                  mlflow=False,
+                  tb=False,
+                  lambda_coord=1.0,
+                  lambda_obj=5.0,
+                  lambda_noobj=1.0,
+                  lambda_class=1.0,
+                  max_outputs=2,
+                  step=None,
+                  step_float=None,
+                  file_writer=None):
+        groundtruth_annotations_list = []
+        img_list = []
+        detections_list = []
+        img_id = 1
+        obj_id = 1
+        val_loss_avg = tf.keras.metrics.Mean()
+        for elements in dataset:
+            images = elements[0]
+            labels = elements[1]
+            img_width = int(
+                tf.cast(tf.shape(images)[1], dtype=tf.float32).numpy())
+            img_height = int(
+                tf.cast(tf.shape(images)[2], dtype=tf.float32).numpy())
+            # Set training false, so that BN uses the mean and variance of
+            # its moving statistics, learned during training.
+            val_logits = self.model(images, training=False)
+            loss = yolo_loss(
+                labels,
+                val_logits,
+                self.anchors,
+                lambda_coord=lambda_coord,
+                lambda_obj=lambda_obj,
+                lambda_noobj=lambda_noobj,
+                lambda_class=lambda_class,
+                iou_threshold=iou_threshold,
+                step=step,
+                is_training=False)
+            val_loss_avg.update_state(loss)
+
+            if tb:
+                # Validation prediction
+                # Bounding box prediction to tensorboard.
+                tb_prediction_val = self._prediction_to_tensorboard(
+                    images, val_logits, max_outputs, step, iou_threshold,
+                    class_conf_threshold)
+                with file_writer.as_default():
+                    tf.summary.image(
+                        "validation", tb_prediction_val, step=step)
+
+            # Coco mAP validation metrics.
+            # Effective batch size. Last batch could contain fewer elements.
+            eff_bs = tf.cast(
+                tf.shape(images)[0], dtype=tf.int32).numpy().item()
+            bboxs, cat_ids, scores = self.coco_metric_format(
+                val_logits, eff_bs, class_conf_threshold, iou_threshold, False)
+            # Iterate over all detections per image.
+            img_id_tmp = img_id
+            for bbox_img, cat_id_img, score_img in zip(bboxs, cat_ids, scores):
+                # Iterate over all detections in given img.
+                for bbox, cat_id, score in zip(bbox_img, cat_id_img,
+                                               score_img):
+                    # bbox is: [xmin,ymin,xmax,ymax]
+                    # bbox : [x,y,w,h]
+                    xmin = bbox[0]
+                    ymin = bbox[1]
+                    xmax = bbox[2]
+                    ymax = bbox[3]
+
+                    # Center coordinate.
+                    x = 0.5 * (xmin + xmax) * img_width
+                    y = 0.5 * (ymin + ymax) * img_height
+                    width = (xmax - xmin) * img_width
+                    height = (ymax - ymin) * img_height
+                    detect_item = {
+                        'image_id': img_id_tmp,
+                        'category_id': int(cat_id.item()),
+                        'bbox': [x, y, width, height],
+                        'score': score
+                    }
+                    detections_list.append(detect_item)
+
+                img_id_tmp += 1
+
+            bboxs, cat_ids, scores = self.coco_metric_format(
+                labels, eff_bs, 0, iou_threshold, True)
+            # Iterate over all objects in labels.
+            img_id_tmp = img_id
+            for bbox_img, cat_id_img, score_img in zip(bboxs, cat_ids, scores):
+                # Iterate over all detections in given img.
+                for bbox, cat_id, score in zip(bbox_img, cat_id_img,
+                                               score_img):
+                    # bbox is: [xmin,ymin,xmax,ymax]
+                    # bbox : [x,y,w,h]
+                    xmin = bbox[0]
+                    ymin = bbox[1]
+                    xmax = bbox[2]
+                    ymax = bbox[3]
+
+                    # Center coordinate.
+                    x = 0.5 * (xmin + xmax) * img_width
+                    y = 0.5 * (ymin + ymax) * img_height
+                    width = (xmax - xmin) * img_width
+                    height = (ymax - ymin) * img_height
+                    grountruth_item = {
+                        'id': obj_id,
+                        'image_id': img_id_tmp,
+                        'category_id': int(cat_id.item()),
+                        'bbox': [x, y, width, height],
+                        'area': width * height,
+                        'iscrowd': 0
+                    }
+                    obj_id += 1
+                    groundtruth_annotations_list.append(grountruth_item)
+                img_list.append({"id": img_id_tmp})
+                img_id_tmp += 1
+            img_id += eff_bs
+
+        groundtruth_dict = {
+            'annotations': groundtruth_annotations_list,
+            'images': img_list,
+            'categories': self.category_list
+        }
+        #print("dl", detections_list)
+        #print("-----------------")
+        #print("dl_len", len(detections_list))
+        #print("gt_len", len(groundtruth_annotations_list))
+        #print("-----------------")
+        #print("gt", groundtruth_dict)
+        metrics = compute_coco_metrics(groundtruth_dict, detections_list)
+        if tb:
+            with file_writer.as_default():
+                tf.summary.scalar(
+                    "total_loss", val_loss_avg.result(), step=step)
+                for key in metrics:
+                    tf.summary.scalar(key, metrics[key], step=step)
+            file_writer.flush()
+
+        if mlflow:
+            # Convert to native python type (db).
+            mlflow.log_metric(
+                "mAP50IOU",
+                metrics["Precision/mAP@.50IOU"].item(),
+                step=step_float)
+            mlflow.log_metric(
+                "AR100", metrics["Recall/AR@100"].item(), step=step_float)
+            mlflow.log_metric(
+                "val_loss",
+                val_loss_avg.result().numpy().item(),
+                step=step_float)
+        print("...Done - loss {}\n".format(
+            round(val_loss_avg.result().numpy().item(), 4)))
\ No newline at end of file
